//+------------------------------------------------------------------+
//|                                          SignalEquidChannel1.mqh |
///|                                          Copyright 2016, denkir |
//|                             https://www.mql5.com/ru/users/denkir |
//+------------------------------------------------------------------+
#property copyright "Copyright 2016, denkir"
#property link      "https://www.mql5.com/ru/users/denkir"
//--- include
#include <Expert\ExpertSignal.mqh>
#include "..\CFractalPoint.mqh"
//+------------------------------------------------------------------+
//| Class CSignalEquidChannel                                        |
//| Purpose:             |
//|          .                                                 |
//|   CExpertSignal.                                    |
//+------------------------------------------------------------------+
class CSignalEquidChannel : public CExpertSignal
  {
protected:
   CiCustom          m_equi_chs;          // - "EquidistantChannels"   
   //---  
   int               m_prev_frac_num;     //  
   bool              m_to_plot_fracs;     //  ?
   int               m_bars_beside;       //  /  
   int               m_bars_between;      //  
   ENUM_RELEVANT_EXTREMUM m_relevant_pnt; //  
   int               m_line_width;        //  
   bool              m_to_log;            //  ?
   double            m_pnt_in;            //  ,
   double            m_pnt_out;           //  ,
   bool              m_on_start;          //    
   //--- 
   double            m_base_low_price;    //   
   double            m_base_high_price;   //   
   double            m_upper_zone[2];     //  : [0]- , [1]-  
   double            m_lower_zone[2];     //  
   datetime          m_last_ch_time;      //    

   //--- ""   (0-100)
   int               m_pattern_0;         //  "    - buy,  - sell"

   //--- === Methods === --- 
public:
   //--- /
   void              CSignalEquidChannel(void);
   void             ~CSignalEquidChannel(void){};
   //---    
   void              PrevFracNum(int _prev_frac_num)   {m_prev_frac_num=_prev_frac_num;}
   void              ToPlotFracs(bool _to_plot)        {m_to_plot_fracs=_to_plot;}
   void              BarsBeside(int _bars_beside)      {m_bars_beside=_bars_beside;}
   void              BarsBetween(int _bars_between)    {m_bars_between=_bars_between;}
   void              RelevantPoint(ENUM_RELEVANT_EXTREMUM _pnt) {m_relevant_pnt=_pnt;}
   void              LineWidth(int _line_wid)          {m_line_width=_line_wid;}
   void              ToLog(bool _to_log)               {m_to_log=_to_log;}
   void              PointsOutside(double _out_pnt)    {m_pnt_out=_out_pnt;}
   void              PointsInside(double _in_pnt)      {m_pnt_in=_in_pnt;}
   void              SignalOnStart(bool _on_start)     {m_on_start=_on_start;}
   //---   ""  
   void              Pattern_0(int _val) {m_pattern_0=_val;}
   //---   
   virtual bool      ValidationSettings(void);
   //---     
   virtual bool      InitIndicators(CIndicators *indicators);
   //---  ,    
   virtual int       LongCondition(void);
   virtual int       ShortCondition(void);
   virtual double    Direction(void);
   //---
protected:
   //---   
   bool              InitCustomIndicator(CIndicators *indicators);
   //-     
   double            Upper(int ind)
     {
      return(m_equi_chs.GetData(2,ind));
     }
   //-     
   double            Lower(int ind)
     {
      return(m_equi_chs.GetData(3,ind));
     }
   //-    
   double            NewChannel(int ind) {return(m_equi_chs.GetData(4,ind));}
  };
//+------------------------------------------------------------------+
//|                                                       |
//+------------------------------------------------------------------+  
void CSignalEquidChannel::CSignalEquidChannel(void)
  {
//--- initialization of protected data
   m_used_series=USE_SERIES_OPEN+USE_SERIES_HIGH+USE_SERIES_LOW+
                 USE_SERIES_CLOSE+USE_SERIES_SPREAD+USE_SERIES_TIME;
//---
   m_base_low_price=m_base_high_price=0.;
   m_pnt_in=m_pnt_out=0.;
   m_last_ch_time=0;
   ArrayInitialize(m_upper_zone,0.);
   ArrayInitialize(m_lower_zone,0.);
  }
//+------------------------------------------------------------------+
//|                                |
//+------------------------------------------------------------------+
bool CSignalEquidChannel::ValidationSettings(void)
  {
//--- validation settings of additional filters
   if(!CExpertSignal::ValidationSettings())
      return false;
//---  
   if(m_symbol==NULL)
      return false;
//--- initial data checks
   if((m_prev_frac_num>3) || (m_prev_frac_num<0))
     {
      PrintFormat(__FUNCTION__+":       3   0!");
      return false;
     }
   if(m_bars_beside<1)
     {
      PrintFormat(__FUNCTION__+":  /      1!");
      return false;
     }
   if(m_bars_between<1)
     {
      PrintFormat(__FUNCTION__+":       1!");
      return false;
     }
   if(m_line_width<1)
     {
      PrintFormat(__FUNCTION__+":       1!");
      return false;
     }
   if(m_pnt_in<0.)
     {
      PrintFormat(__FUNCTION__+":       0.0!");
      return false;
     }
   if(m_pnt_out<0.)
     {
      PrintFormat(__FUNCTION__+":       0.0!");
      return false;
     }
//--- ok
   return true;
  }
//+------------------------------------------------------------------+
//|                                               |
//+------------------------------------------------------------------+
bool CSignalEquidChannel::InitIndicators(CIndicators *indicators)
  {
//---      
   if(!CExpertSignal::InitIndicators(indicators))
      return false;
//---     
   if(!this.InitCustomIndicator(indicators))
      return false;
//--- ok
   return true;
  }
//+------------------------------------------------------------------+
//|                               |
//+------------------------------------------------------------------+
bool CSignalEquidChannel::InitCustomIndicator(CIndicators *indicators)
  {
//---    
   if(!indicators.Add(GetPointer(m_equi_chs)))
     {
      PrintFormat(__FUNCTION__+": error adding object");
      return false;
     }
//---   
   MqlParam parameters[8];
   parameters[0].type=TYPE_STRING;
   parameters[0].string_value="EquidistantChannels.ex5";
   parameters[1].type=TYPE_INT;
   parameters[1].integer_value=m_prev_frac_num;   // 1)  
   parameters[2].type=TYPE_BOOL;
   parameters[2].integer_value=m_to_plot_fracs;   // 2)  ?
   parameters[3].type=TYPE_INT;
   parameters[3].integer_value=m_bars_beside;     // 3)  /  
   parameters[4].type=TYPE_INT;
   parameters[4].integer_value=m_bars_between;    // 4)  
   parameters[5].type=TYPE_INT;
   parameters[5].integer_value=m_relevant_pnt;    // 5)  
   parameters[6].type=TYPE_INT;
   parameters[6].integer_value=m_line_width;    // 6)  
   parameters[7].type=TYPE_BOOL;
   parameters[7].integer_value=m_to_log;          // 7)  ?

//---  
   if(!m_equi_chs.Create(m_symbol.Name(),_Period,IND_CUSTOM,8,parameters))
     {
      PrintFormat(__FUNCTION__+": error initializing object");
      return false;
     }
//---  
   if(!m_equi_chs.NumBuffers(5))
      return false;
//--- ok
   return true;
  }
//+------------------------------------------------------------------+
//|                                          |
//+------------------------------------------------------------------+
int CSignalEquidChannel::LongCondition(void)
  {
   int result=0;
//---    
   if(m_base_low_price>0.)
      //---       
      if((m_base_low_price<=m_lower_zone[0]) && (m_base_low_price>=m_lower_zone[1]))
        {
         if(IS_PATTERN_USAGE(0))
            result=m_pattern_0;
        }
//---
   return result;
  }
//+------------------------------------------------------------------+
//|                                          |
//+------------------------------------------------------------------+
int CSignalEquidChannel::ShortCondition(void)
  {
   int result=0;
//---    
   if(m_base_high_price>0.)
      //---       
      if((m_base_high_price>=m_upper_zone[0]) && (m_base_high_price<=m_upper_zone[1]))
        {
         if(IS_PATTERN_USAGE(0))
            result=m_pattern_0;
        }
//---
   return result;
  }
//+------------------------------------------------------------------+
//|  ""                             |
//+------------------------------------------------------------------+
double CSignalEquidChannel::Direction(void)
  {
   double result=0.;
//---   
   datetime last_bar_time=this.Time(0);
   bool is_new_channel=(this.NewChannel(0)>0.);
//---     
   if(!m_on_start)
      //---       
      if(m_prev_frac_num==3)
        {
         static datetime last_ch_time=0;
         //---    
         if(is_new_channel)
           {
            last_ch_time=last_bar_time;
            //---   
            if(m_last_ch_time==0)
               //---   ,    
               m_last_ch_time=last_ch_time;
           }
         //---   
         if(m_last_ch_time==last_ch_time)
            return 0.;
         else
         //---  
            m_on_start=true;
        }
//---   
   int actual_bar_idx=this.StartIndex();
//---  
   double upper_vals[2],lower_vals[2]; // [0]-  , [1]- 
   ArrayInitialize(upper_vals,0.);
   ArrayInitialize(lower_vals,0.);
   for(int idx=ArraySize(upper_vals)-1,jdx=0;idx>=0;idx--,jdx++)
     {
      upper_vals[jdx]=this.Upper(actual_bar_idx+idx);
      lower_vals[jdx]=this.Lower(actual_bar_idx+idx);
      if((upper_vals[jdx]==0.) || (lower_vals[jdx]==0.))
         return 0.;
     }
//---  
   double curr_high_pr,curr_low_pr;
   curr_high_pr=this.High(actual_bar_idx);
   curr_low_pr=this.Low(actual_bar_idx);
//---   
   if(curr_high_pr!=EMPTY_VALUE)
      if(curr_low_pr!=EMPTY_VALUE)
        {
         //---  
         m_base_low_price=curr_low_pr;
         m_base_high_price=curr_high_pr;
         //---     
         //---  : [0]- , [1]- 
         this.m_upper_zone[0]=upper_vals[1]-m_pnt_in;
         this.m_upper_zone[1]=upper_vals[1]+m_pnt_out;
         //---  : [0]- , [1]- 
         this.m_lower_zone[0]=lower_vals[1]+m_pnt_in;
         this.m_lower_zone[1]=lower_vals[1]-m_pnt_out;
         //--- 
         for(int jdx=0;jdx<ArraySize(m_lower_zone);jdx++)
           {
            this.m_lower_zone[jdx]=m_symbol.NormalizePrice(m_lower_zone[jdx]);
            this.m_upper_zone[jdx]=m_symbol.NormalizePrice(m_upper_zone[jdx]);
           }
         //--- ,    
         if(this.m_upper_zone[0]<=this.m_lower_zone[0])
            return 0.;
         //--- 
         result=m_weight*(this.LongCondition()-this.ShortCondition());
        }
//---
   return result;
  }
//--- [EOF]
